@@ -15,9 +15,11 @@
  */
 
 #include <cstring>
+#include <string>
 
-#include "com_google_media_codecs_ultrahdr_UltraHDREncoder.h"
+#include "com_google_media_codecs_ultrahdr_UltraHDRCommon.h"
 #include "com_google_media_codecs_ultrahdr_UltraHDRDecoder.h"
+#include "com_google_media_codecs_ultrahdr_UltraHDREncoder.h"
 #include "ultrahdr_api.h"
 
 static_assert(sizeof(void *) <= sizeof(jlong),
@@ -319,6 +321,18 @@ Java_com_google_media_codecs_ultrahdr_UltraHDREncoder_setOutputFormatNative(JNIE
       status.has_detail ? status.detail : "uhdr_enc_set_output_format() returned with error")
 }
 
+extern "C" JNIEXPORT void JNICALL
+Java_com_google_media_codecs_ultrahdr_UltraHDREncoder_setMinMaxContentBoostNative(
+    JNIEnv *env, jobject thiz, jfloat min_content_boost, jfloat max_content_boost) {
+  GET_HANDLE()
+  RET_IF_TRUE(handle == 0, "java/io/IOException", "invalid encoder instance")
+  auto status = uhdr_enc_set_min_max_content_boost((uhdr_codec_private_t *)handle,
+                                                   min_content_boost, max_content_boost);
+  RET_IF_TRUE(status.error_code != UHDR_CODEC_OK, "java/io/IOException",
+              status.has_detail ? status.detail
+                                : "uhdr_enc_set_min_max_content_boost() returned with error")
+}
+
 extern "C" JNIEXPORT void JNICALL
 Java_com_google_media_codecs_ultrahdr_UltraHDREncoder_encodeNative(JNIEnv *env, jobject thiz) {
   GET_HANDLE()
@@ -447,6 +461,18 @@ Java_com_google_media_codecs_ultrahdr_UltraHDRDecoder_setMaxDisplayBoostNative(
                                 : "uhdr_dec_set_out_max_display_boost() returned with error")
 }
 
+extern "C" JNIEXPORT void JNICALL
+Java_com_google_media_codecs_ultrahdr_UltraHDRDecoder_enableGpuAccelerationNative(JNIEnv *env,
+                                                                                  jobject thiz,
+                                                                                  jint enable) {
+  GET_HANDLE()
+  RET_IF_TRUE(handle == 0, "java/io/IOException", "invalid decoder instance")
+  uhdr_error_info_t status = uhdr_enable_gpu_acceleration((uhdr_codec_private_t *)handle, enable);
+  RET_IF_TRUE(
+      status.error_code != UHDR_CODEC_OK, "java/io/IOException",
+      status.has_detail ? status.detail : "uhdr_enable_gpu_acceleration() returned with error")
+}
+
 extern "C" JNIEXPORT void JNICALL
 Java_com_google_media_codecs_ultrahdr_UltraHDRDecoder_probeNative(JNIEnv *env, jobject thiz) {
   GET_HANDLE()
@@ -522,13 +548,41 @@ Java_com_google_media_codecs_ultrahdr_UltraHDRDecoder_getIccNative(JNIEnv *env,
   return data;
 }
 
+extern "C" JNIEXPORT jbyteArray JNICALL
+Java_com_google_media_codecs_ultrahdr_UltraHDRDecoder_getBaseImageNative(JNIEnv *env,
+                                                                         jobject thiz) {
+  GET_HANDLE_VAL(nullptr)
+  uhdr_mem_block_t *baseImgData = uhdr_dec_get_base_image((uhdr_codec_private_t *)handle);
+  RET_VAL_IF_TRUE(baseImgData == nullptr, "java/io/IOException",
+                  "uhdr_dec_probe() is not yet called or it has returned with error", nullptr)
+  jbyteArray data = env->NewByteArray(baseImgData->data_sz);
+  jbyte *dataptr = env->GetByteArrayElements(data, nullptr);
+  std::memcpy(dataptr, baseImgData->data, baseImgData->data_sz);
+  env->ReleaseByteArrayElements(data, dataptr, 0);
+  return data;
+}
+
+extern "C" JNIEXPORT jbyteArray JNICALL
+Java_com_google_media_codecs_ultrahdr_UltraHDRDecoder_getGainMapImageNative(JNIEnv *env,
+                                                                            jobject thiz) {
+  GET_HANDLE_VAL(nullptr)
+  uhdr_mem_block_t *gainmapImgData = uhdr_dec_get_gainmap_image((uhdr_codec_private_t *)handle);
+  RET_VAL_IF_TRUE(gainmapImgData == nullptr, "java/io/IOException",
+                  "uhdr_dec_probe() is not yet called or it has returned with error", nullptr)
+  jbyteArray data = env->NewByteArray(gainmapImgData->data_sz);
+  jbyte *dataptr = env->GetByteArrayElements(data, nullptr);
+  std::memcpy(dataptr, gainmapImgData->data, gainmapImgData->data_sz);
+  env->ReleaseByteArrayElements(data, dataptr, 0);
+  return data;
+}
+
 extern "C" JNIEXPORT void JNICALL
 Java_com_google_media_codecs_ultrahdr_UltraHDRDecoder_getGainmapMetadataNative(JNIEnv *env,
                                                                                jobject thiz) {
   GET_HANDLE()
   RET_IF_TRUE(handle == 0, "java/io/IOException", "invalid decoder instance")
   uhdr_gainmap_metadata_t *gainmap_metadata =
-      uhdr_dec_get_gain_map_metadata((uhdr_codec_private_t *)handle);
+      uhdr_dec_get_gainmap_metadata((uhdr_codec_private_t *)handle);
   RET_IF_TRUE(gainmap_metadata == nullptr, "java/io/IOException",
               "uhdr_dec_probe() is not yet called or it has returned with error")
 #define SET_FLOAT_FIELD(name, val)                                    \
@@ -587,10 +641,10 @@ Java_com_google_media_codecs_ultrahdr_UltraHDRDecoder_getDecodedImageNative(JNIE
 }
 
 extern "C" JNIEXPORT jbyteArray JNICALL
-Java_com_google_media_codecs_ultrahdr_UltraHDRDecoder_getGainMapImageNative(JNIEnv *env,
-                                                                            jobject thiz) {
+Java_com_google_media_codecs_ultrahdr_UltraHDRDecoder_getDecodedGainMapImageNative(JNIEnv *env,
+                                                                                   jobject thiz) {
   GET_HANDLE_VAL(nullptr)
-  uhdr_raw_image_t *gainmapImg = uhdr_get_gain_map_image((uhdr_codec_private_t *)handle);
+  uhdr_raw_image_t *gainmapImg = uhdr_get_decoded_gainmap_image((uhdr_codec_private_t *)handle);
   RET_VAL_IF_TRUE(gainmapImg == nullptr, "java/io/IOException",
                   "uhdr_decode() is not yet called or it has returned with error", nullptr)
   int bpp = gainmapImg->fmt == UHDR_IMG_FMT_32bppRGBA8888 ? 4 : 1;
@@ -612,3 +666,15 @@ Java_com_google_media_codecs_ultrahdr_UltraHDRDecoder_resetNative(JNIEnv *env, j
   RET_IF_TRUE(handle == 0, "java/io/IOException", "invalid decoder instance")
   uhdr_reset_decoder((uhdr_codec_private_t *)handle);
 }
+
+extern "C" JNIEXPORT jstring JNICALL
+Java_com_google_media_codecs_ultrahdr_UltraHDRCommon_getVersionStringNative(JNIEnv *env,
+                                                                            jclass clazz) {
+  std::string version{"v" UHDR_LIB_VERSION_STR};
+  return env->NewStringUTF(version.c_str());
+}
+
+extern "C" JNIEXPORT jint JNICALL
+Java_com_google_media_codecs_ultrahdr_UltraHDRCommon_getVersionNative(JNIEnv *env, jclass clazz) {
+  return UHDR_LIB_VERSION;
+}
