@@ -29,19 +29,29 @@
 namespace ultrahdr {
 
 // Default configurations
-// Map is quarter res / sixteenth size
-static const size_t kMapDimensionScaleFactorDefault = 4;
+// gainmap image downscale factor
+static const size_t kMapDimensionScaleFactorDefault = 1;
+static const size_t kMapDimensionScaleFactorAndroidDefault = 4;
+
+// JPEG compress quality (0 ~ 100) for base image
+static const int kBaseCompressQualityDefault = 95;
 
 // JPEG compress quality (0 ~ 100) for gain map
-static const int kMapCompressQualityDefault = 85;
+static const int kMapCompressQualityDefault = 95;
+static const int kMapCompressQualityAndroidDefault = 85;
 
 // Gain map calculation
-static const bool kUseMultiChannelGainMapDefault = false;
+static const bool kUseMultiChannelGainMapDefault = true;
+static const bool kUseMultiChannelGainMapAndroidDefault = false;
+
+// encoding preset
+static const uhdr_enc_preset_t kEncSpeedPresetDefault = UHDR_USAGE_BEST_QUALITY;
+static const uhdr_enc_preset_t kEncSpeedPresetAndroidDefault = UHDR_USAGE_REALTIME;
+
 // Default gamma value for gain map
 static const float kGainMapGammaDefault = 1.0f;
 
 // The current JPEGR version that we encode to
-static const char* const kGainMapVersion = "1.0";
 static const char* const kJpegrVersion = "1.0";
 
 /*
@@ -74,11 +84,12 @@ typedef struct jpegr_info_struct* jr_info_ptr;
 class JpegR {
  public:
   JpegR(void* uhdrGLESCtxt = nullptr,
-        size_t mapDimensionScaleFactor = kMapDimensionScaleFactorDefault,
-        int mapCompressQuality = kMapCompressQualityDefault,
-        bool useMultiChannelGainMap = kUseMultiChannelGainMapDefault,
-        float gamma = kGainMapGammaDefault, uhdr_enc_preset_t preset = UHDR_USAGE_REALTIME,
-        float minContentBoost = FLT_MIN, float maxContentBoost = FLT_MAX);
+        size_t mapDimensionScaleFactor = kMapDimensionScaleFactorAndroidDefault,
+        int mapCompressQuality = kMapCompressQualityAndroidDefault,
+        bool useMultiChannelGainMap = kUseMultiChannelGainMapAndroidDefault,
+        float gamma = kGainMapGammaDefault,
+        uhdr_enc_preset_t preset = kEncSpeedPresetAndroidDefault, float minContentBoost = FLT_MIN,
+        float maxContentBoost = FLT_MAX);
 
   /*!\brief Encode API-0.
    *
